@@ -1,6 +1,6 @@
 # VC  -  underpinnings of a VC-agnostic diff and commit tool.
 
-# Copyright (C) 2006-2013 Free Software Foundation, Inc.
+# Copyright (C) 2006-2017 Free Software Foundation, Inc.
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,6 +42,7 @@ use constant
     HG  => 'hg',
     SVN => 'svn',
     BZR => 'bzr',
+    DARCS => 'darcs',
   };
 
 my $vc_cmd =
@@ -56,8 +57,9 @@ my $vc_cmd =
    GIT() =>
    {
     AUTHOR_FMT => '--author=%s',
-    DIFF_COMMAND => [qw(git diff -B -C HEAD --)],
-    DIFF_PRISTINE => [qw(git diff --no-ext-diff -B -C HEAD --)],
+    DIFF_COMMAND => [qw(git diff --ignore-submodules=all -B -C HEAD --)],
+    DIFF_PRISTINE => [qw(git diff --ignore-submodules=all --no-ext-diff
+                         -B -C HEAD --)],
     VALID_DIFF_EXIT_STATUS => {0 => 1},
     COMMIT_COMMAND => [qw(git commit -q -F)],
     # is-version-controlled-file: true, if "git cat-file -t HEAD:$file"
@@ -84,6 +86,13 @@ my $vc_cmd =
     VALID_DIFF_EXIT_STATUS => {0 => 1, 1 => 1},
     COMMIT_COMMAND => [qw(bzr ci -q -F)],
    },
+   DARCS() => # aka bazaar
+   {
+    AUTHOR_FMT => '--author=%s',
+    DIFF_COMMAND => [qw(darcs diff --)],
+    VALID_DIFF_EXIT_STATUS => {0 => 1},
+    COMMIT_COMMAND => [qw(darcs record -q -a --look-for-adds --logfile)],
+   },
   };
 
 #########
@@ -129,6 +138,8 @@ sub new($%)
 	$self->{name} = BZR;
       } elsif (-d "$d/.hg") {
 	$self->{name} = HG;
+      } elsif (-d "$d/_darcs") {
+	$self->{name} = DARCS;
       }
 
       my ($dev, $ino, undef) = stat $d;
